﻿<# 
.SYNOPSIS 
	Clones an existing Lync Server 2013/Skype for Business Server 2015 policy into a new policy with all existing settings.

.DESCRIPTION 
	Clones an existing Lync Server 2013/Skype for Business Server 2015 policy into a new policy with all existing settings. The new policy can then be further customized if needed.

.NOTES 
	Version							  : 1.3 - See changelog at http://www.ucunleashed.com/2302
	Wish list						  : Better error trapping
	Rights Required			  : CsAdministrator for some functions
	Sched Task Required	  : No
	Lync/Skype4B Version	: 2013 (tested through CU4), Skype for Business Server 2015
	PowerShell Version	  : 3.0
	Author/Copyright      : © Pat Richard, Office Servers and Services (Skype for Business) MVP - All Rights Reserved
	Email/Blog/Twitter	  : pat@innervation.com 	https://www.ucunleashed.com @patrichard
	Donations             : https://www.paypal.me/PatRichard
	Dedicated Post			  : https://www.ucunleashed.com/2300
	Disclaimer            : You running this script means you won't blame author(s) if this breaks your stuff. This script is
                          provided AS IS without warranty of any kind. Author(s) disclaim all implied warranties including,
                          without limitation, any implied warranties of merchantability or of fitness for a particular
                          purpose. The entire risk arising out of the use or performance of the sample scripts and
                          documentation remains with you. In no event shall author(s) be liable for any damages whatsoever
                          (including, without limitation, damages for loss of business profits, business interruption,
                          loss of business information, or other pecuniary loss) arising out of the use of or inability
                          to use the script or documentation. Neither this script, nor any part of it other than those
                          parts that are explicitly copied from others, may be republished without author(s) express written 
                          permission.
	Acknowledgements 		  : Brian Desmond (@brdesmond) for pointing out the InnerText method: http://msdn.microsoft.com/en-us/library/system.xml.xmlelement(v=vs.100).aspx
											  : Using XML to copy policies: http://blogs.technet.com/b/nexthop/archive/2011/03/21/copypolicies.aspx
											  : Changing XML data: http://powershell.com/cs/blogs/tobias/archive/2009/02/02/xml-part-2-write-add-and-change-xml-data.aspx
	Assumptions					  : ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
	Limitations					  : Only works on Windows Server 2012. This is by design. There are no plans to support Windows Server 2008 R2.									
	Known issues				  : None yet, but I'm sure you'll find some!

.LINK 
	https://www.ucunleashed.com/2300

.EXAMPLE
	.\New-CsClonedPolicy.ps1 -PolicyType ConferencingPolicy -SourcePolicyName "global" -TargetPolicyName "New Policy"

	Description
	-----------
	Clones the Conferencing Policy called "global" into a new user level policy called "New Policy"

.EXAMPLE
	.\New-CsClonedPolicy.ps1 -PolicyType ConferencingPolicy -SourcePolicyName "global" -TargetPolicyName "Site:Redmond"

	Description
	-----------
	Clones the Conferencing Policy called "global" into a new site level for the Lync/Skype for Business site "Redmond". 

.INPUTS
	None. You cannot pipe objects to this script.
#>
#Requires -Version 3.0
#Requires -Modules Lync

[CmdletBinding(SupportsShouldProcess, SupportsPaging)]
param(
  # Folder location of the exported file while cloning the policy.
  [Parameter(ValueFromPipelineByPropertyName)]
	[ValidateNotNullOrEmpty()]
	[string]$ExportFolder = "$env:UserProfile\desktop",
  
  # Name of temp XML file used while the policy is being cloned. It is automatically deleted when finished.
  [Parameter(ValueFromPipelineByPropertyName)]
	[ValidateNotNullOrEmpty()]
	[string]$ExportFile = "ExportedPolicy.xml",
  
  # Defines the policy that will be copied.
  [Parameter(ValueFromPipelineByPropertyName, Mandatory)]
	[ValidateNotNullOrEmpty()]
	[string]$SourcePolicyName,
  
	# Defines the name of the new policy. This policy cannot already exist. By default, specifying a value will create a user level policy. Specifying a name preceeded by "site:" will create a site level policy for that site.
  [Parameter(ValueFromPipelineByPropertyName, Mandatory)]
	[ValidateNotNullOrEmpty()]
	[string]$TargetPolicyName,
  
  # Defines the type of policy being cloned. Valid values are "ArchivingPolicy", "ClientPolicy", "ClientVersionPolicy", "ConferencingPolicy", "ExternalAccessPolicy", "HostedVoicemailPolicy", "LocationPolicy", "MobilityPolicy", "NetworkInterSitePolicy", "PersistentChatPolicy", "PinPolicy", "PresencePolicy", "UserServicesPolicy", "VoicePolicy", "VoiceRoutingPolicy"
  [Parameter(ValueFromPipelineByPropertyName, Mandatory)]
	[ValidateNotNullOrEmpty()]
	[ValidateSet("ArchivingPolicy", "ClientPolicy", "ClientVersionPolicy", "ConferencingPolicy", "ExternalAccessPolicy", "HostedVoicemailPolicy", "LocationPolicy", "MobilityPolicy", "NetworkInterSitePolicy", "PersistentChatPolicy", "PinPolicy", "PresencePolicy", "UserServicesPolicy", "VoicePolicy", "VoiceRoutingPolicy")]
	[string]$PolicyType,
	
	# Description for the new policy.
	[Parameter(ValueFromPipelineByPropertyName)]
	[string]$Description = "Cloned from the policy `'$SourcePolicyName`' on $(Get-Date -Format f). Created by New-CsClonedPolicy.ps1 (see https://www.ucunleashed.com/2300)",
	
	# Defines the location for any downloaded files. Defaults to "c:\_install". Additionally, log files generated by this script are located in a subfolder of TargetFolder called "logs". TargetFolder does not support paths with spaces, but does support non-hidden UNC paths.
  [Parameter(ValueFromPipelineByPropertyName)]
  [ValidateNotNullOrEmpty()]
  [ValidatePattern('(?:[a-zA-Z]\:|\\\\[]\w\.+|\w-\\[\w.]+)\\(?:[\w]+\\)*\w[\w.]+')]
  [string] $TargetFolder = "$env:SystemDrive\_Install"
)
[string] $ScriptVersion = "1.3"
[int] $Article = 2300
[string] $LogPath = "$TargetFolder\logs\$env:ComputerName" + " {0:yyyy-MM-dd hh-mmtt}.log" -f (Get-Date)
[string] $LogDivider = "+------------------------------+"

#region functions
function Get-UpdateInfo {
<#
.SYNOPSIS
  Queries an online XML source for version information to determine if a new version of the script is available.

.DESCRIPTION
  Queries an online XML source for version information to determine if a new version of the script is available.

.NOTES
  Version               : 1.2 - See changelog at http://www.ehloworld.com/3168 for fixes & changes introduced with each version
  Wish list             : Better error trapping
  Rights Required       : N/A
  Sched Task Required   : No
  Lync/Skype4B Version  : N/A
  Author/Copyright      : © Pat Richard, Office Servers and Services (Skype for Business) MVP - All Rights Reserved
  Email/Blog/Twitter    : pat@innervation.com  https://ucunleashed.com  @patrichard
  Donations             : https://www.paypal.me/PatRichard
  Dedicated Post        : http://www.ehloworld.com/3168
  Disclaimer            : You running this script means you won't blame author(s) if this breaks your stuff. This script is
                          provided AS IS without warranty of any kind. Author(s) disclaim all implied warranties including,
                          without limitation, any implied warranties of merchantability or of fitness for a particular
                          purpose. The entire risk arising out of the use or performance of the sample scripts and
                          documentation remains with you. In no event shall author(s) be liable for any damages whatsoever
                          (including, without limitation, damages for loss of business profits, business interruption,
                          loss of business information, or other pecuniary loss) arising out of the use of or inability
                          to use the script or documentation. Neither this script, nor any part of it other than those
                          parts that are explicitly copied from others, may be republished without author(s) express written 
                          permission.
  Acknowledgements      : Reading XML files 
                          http://stackoverflow.com/questions/18509358/how-to-read-xml-in-powershell
                          http://stackoverflow.com/questions/20433932/determine-xml-node-exists
  Assumptions           : ExecutionPolicy of AllSigned (recommended), RemoteSigned, or Unrestricted (not recommended)
  Limitations           : 
  Known issues          : 

.LINK
  http://www.ehloworld.com/3168

.EXAMPLE
  Get-UpdateInfo -Article 123

  Description
  -----------
  Runs function to check for updates to article/script 123.

.INPUTS
  None. You cannot pipe objects to this script.
#>
  [CmdletBinding(SupportsShouldProcess, SupportsPaging)]
  param (
    [string] $article
  )
  [bool] $HasInternetAccess = ([Activator]::CreateInstance([Type]::GetTypeFromCLSID([Guid]'{DCB00C01-570F-4A9B-8D69-199FDBA5723B}')).IsConnectedToInternet)
  if ($HasInternetAccess){
    [xml] $xml = (New-Object System.Net.WebClient).DownloadString("http://www.ehloworld.com/downloads/version.xml")
    [string] $Ga = ($xml.catalog.article | Where-Object {$_.id -eq $article}).version
    if (($xml.catalog.article | Where-Object {$_.id -eq $article}).versionInfo){
      [string] $changelog = "This version includes: " + ($xml.catalog.article | Where-Object {$_.id -eq $article}).versionInfo
    }
    if ($Ga -gt $ScriptVersion){
      Write-Log -Level Warn -Message "Outdated version. Version $Ga is latest version. Prompting user to upgrade." -NoConsole
      if ((New-Popup -Message "Version $Ga is now available. $changelog `n`nWould you like to download it?" -Title "New version available!" -Buttons YesNo -Icon Question) -eq 6){
        Write-Log -Message "User elected to download latest version." -Indent 1 -NoConsole
        Write-Log -Message "Opening Internet Explorer and going to http://www.ehloworld.com/$article#downloads." -NoConsole -Indent 1
        Start-Process "http://www.ehloworld.com/$article#downloads"
        Write-Log -Level Warn -Message "Script is exiting. Please start the new version of the script once you've downloaded it." -NoLog
        Exit
      }else{
        Write-Log -Level Warn -Message "User elected NOT to download latest version." -Indent 1 -NoConsole
      }
    }elseif ($Ga -eq $ScriptVersion){
      Write-Log -Message "Current version. Version $Ga is latest version." -NoConsole
    }elseif (-Not ($Ga)) {
      # Write-Log -Level Warn -Message "Possibly regressed version. Version $Ga is latest version." -NoConsole
      Write-Log -Level Warn -Message "No version info available" -NoConsole
    }
  }else{
    Write-Log -Level Warn -Message "Unable to check online for update info." -NoConsole
  }
} # end function function Get-UpdateInfo

function Update-XMLData{
	[CmdletBinding(SupportsShouldProcess)]
	param()
	if (-not(Test-Path "$ExportFolder\$ExportFile")){
		Write-Log -Level Error -Message "Unable to export policy settings for `"$SourcePolicyName`" to $ExportFolder\$ExportFile"
		break
	}
	Write-Log -Level Info -Message "Updating XML data" -NoConsole
	$xml = New-Object XML
	$xml.Load("$ExportFolder\$ExportFile")
	($xml.Objs.Obj.Props.S | Where-Object {$_.N -eq "Identity"}).InnerText = $TargetPolicyName
	if ($xml.Objs.Obj.Props.S | Where-Object {$_.N -eq "Name"}){
		($xml.Objs.Obj.Props.S | Where-Object {$_.N -eq "Name"}).InnerText = $TargetPolicyName
	}
	Write-Log -Level Info -Message "Saving updated XML data" -NoConsole
	$xml.Save("$ExportFolder\$ExportFile")
} # end function Update-XMLData

function Write-Log {
  <#
  .SYNOPSIS
    Extensive function to write data to either the console screen, a log file, and/or a Windows event log.

  .DESCRIPTION
    Extensive function to write data to either the console screen, a log file, and/or a Windows event log. Data can be written as info, warning, error, and includes indentation, time stamps, etc.

  .NOTES
    Version               : 2.9
    Wish list             : Better error trapping
    Rights Required       : Local administrator on server
    Sched Task Required   : No
    Lync/Skype4B Version  : N/A
    Author/Copyright      : © Pat Richard, Office Servers and Services (Skype for Business) MVP - All Rights Reserved
    Email/Blog/Twitter    : pat@innervation.com  https://ucunleashed.com  @patrichard
    Donations             : https://www.paypal.me/PatRichard
    Dedicated Post        :
    Disclaimer            : You running this script means you won't blame author(s) if this breaks your stuff. This script is
                            provided AS IS without warranty of any kind. Author(s) disclaim all implied warranties including,
                            without limitation, any implied warranties of merchantability or of fitness for a particular
                            purpose. The entire risk arising out of the use or performance of the sample scripts and
                            documentation remains with you. In no event shall author(s) be liable for any damages whatsoever
                            (including, without limitation, damages for loss of business profits, business interruption,
                            loss of business information, or other pecuniary loss) arising out of the use of or inability
                            to use the script or documentation. Neither this script, nor any part of it other than those
                            parts that are explicitly copied from others, may be republished without author(s) express written 
                            permission.
    Acknowledgements      : Test for log names and sources
                            http://powershell.com/cs/blogs/tips/archive/2013/06/10/testing-event-log-names-and-sources.aspx
    Assumptions           : ExecutionPolicy of AllSigned (recommended), RemoteSigned, or Unrestricted (not recommended)
    Limitations           :
    Known issues          : None yet, but I'm sure you'll find some!

  .LINK

  .EXAMPLE
    .\

    Description
    -----------


  .INPUTS
    None. You cannot pipe objects to this script.
  #>
  [CmdletBinding(SupportsShouldProcess, SupportsPaging)]
  param(
    # The type of message to be logged. Alias is 'type'.
    [Parameter(Position = 0, ValueFromPipeline, ValueFromPipelineByPropertyName)]
    [ValidateSet("Error", "Warn", "Info")]
    [ValidateNotNullOrEmpty()]
    [string] $level = "Info",

    # The message to be logged.
    [Parameter(Position = 1, ValueFromPipeline, ValueFromPipelineByPropertyName, Mandatory, HelpMessage = "No message specified.")]
    [ValidateNotNullOrEmpty()]
    [string] $Message,

    # Specifies that $message should not the sent to the log file.
    [Parameter(Position = 2, ValueFromPipelineByPropertyName)]
    [switch] $NoLog,

    # Specifies to not display the message to the console.
    [Parameter(Position = 3, ValueFromPipelineByPropertyName)]
    [switch] $NoConsole,

    # The number of spaces to indent the message in the log file.
    [Parameter(Position = 4, ValueFromPipelineByPropertyName)]
    [ValidateRange(1,30)]
    [ValidateNotNullOrEmpty()]
    [Int16] $Indent = 0,

    # Specifies what color the text should be be displayed on the console. Ignored when switch 'NoConsoleOut' is specified.
    [Parameter(Position = 5, ValueFromPipelineByPropertyName)]
    [ValidateSet("Black", "DarkMagenta", "DarkRed", "DarkBlue", "DarkGreen", "DarkCyan", "DarkYellow", "Red", "Blue", "Green", "Cyan", "Magenta", "Yellow", "DarkGray", "Gray", "White")]
    [ValidateNotNullOrEmpty()]
    [String] $ConsoleForeground = 'White',

    # Existing log file is deleted when this is specified. Alias is 'Overwrite'.
    [Parameter(Position = 6, ValueFromPipelineByPropertyName)]
    [Switch] $Clobber,

    # The name of the system event log, e.g. 'Application'. Note that writing to the system event log requires elevated permissions.
    [Parameter(Position = 7, ValueFromPipelineByPropertyName)]
    [ValidateSet("Application","System","Security","Lync Server","Microsoft Office Web Apps")]
    [ValidateNotNullOrEmpty()]
    [String] $EventLogName,

    # The name to appear as the source attribute for the system event log entry. This is ignored unless 'EventLogName' is specified.
    [Parameter(Position = 8, ValueFromPipelineByPropertyName)]
    [ValidateNotNullOrEmpty()]
    [String] $EventSource = $($MyInvocation.ScriptName).Name,

    # The ID to appear as the event ID attribute for the system event log entry. This is ignored unless 'EventLogName' is specified.
    [Parameter(Position = 9, ValueFromPipelineByPropertyName)]
    [ValidateRange(1,65535)]
    [ValidateNotNullOrEmpty()]
    [Int32] $EventID = 1,

    # The text encoding for the log file. Default is ASCII.
    [Parameter(Position = 10, ValueFromPipelineByPropertyName)]
    [ValidateSet("Unicode","Byte","BigEndianUnicode","UTF8","UTF7","UTF32","ASCII","Default","OEM")]
    [ValidateNotNullOrEmpty()]
    [String] $LogEncoding = "ASCII"
  ) # end of param block
  try {
    [string]$LogFolder = Split-Path $LogPath -Parent
    if (-Not(Test-Path -Path $LogFolder)){New-Item $LogFolder -type Directory | Out-Null}
    $msg = "{0} : {1} : {2}{3}" -f (Get-Date -Format "yyyy-MM-dd HH:mm:ss"), $Level.ToUpper(), ("  " * $Indent), $Message
    if (-Not($NoConsole)){
      switch ($level) {
        "Error" {$Host.UI.WriteErrorLine("$Message")}
        "Warn" {Write-Warning $Message}
        "Info" {Write-Host $Message -ForegroundColor $ConsoleForeground}
      }
    }
    if (-Not($NoLog)){
      if ($Clobber) {
        $msg | Out-File -FilePath $LogPath -Encoding $LogEncoding -Force
      } else {
        $msg | Out-File -FilePath $LogPath -Encoding $LogEncoding -Append
      }
    }

    # http://social.technet.microsoft.com/Forums/en-US/winserverpowershell/thread/e172f039-ce88-4c9f-b19a-0dd6dc568fa0/

    if ($EventLogName) {
      if (-Not $EventSource) {
        [string] $EventSource = $([IO.FileInfo] $MyInvocation.ScriptName).Name
      }

      if(-Not [Diagnostics.EventLog]::SourceExists($EventSource)) {
        [Diagnostics.EventLog]::CreateEventSource($EventSource, $EventLogName)
      }

      switch ($Level) {
        "Error" {$EntryType = "Error"}
        "Warn"  {$EntryType = "Warning"}
        "Info"  {$EntryType = "Information"}
        Default  {$EntryType = "Information"}
      }
      Write-EventLog -LogName $EventLogName -Source $EventSource -EventId 1 -EntryType $EntryType -Message $Message
    }
    $msg = ""
  } # end try
  catch {
    Throw "Failed to create log entry in: '$LogPath'. The error was: '$_'."
  } # end catch
} # end function Write-Log
#endregion functions

Write-Log -Level Info -Message "Version      : $ScriptVersion" -NoConsole

Get-UpdateInfo -Article $Article

Write-Log -Level Info -Message $LogDivider -NoConsole
Write-Log -Level Info -Message "Source policy: $SourcePolicyName" -NoConsole
Write-Log -Level Info -Message "Target policy: $TargetPolicyName" -NoConsole
Write-Log -Level Info -Message "Policy type  : $PolicyType" -NoConsole
Write-Log -Level Info -Message $LogDivider -NoConsole

[string] $get = "Get-Cs$PolicyType -Identity `"$SourcePolicyName`" | Export-Clixml -Path `"$ExportFolder\$ExportFile`""
[string] $set = "Import-Clixml -Path `"$ExportFolder\$ExportFile`" | Set-Cs$PolicyType"
if ($PolicyType -ne "UserServicesPolicy"){$set += " -Description `"$Description`""}
Write-Log -Level Info -Message "Exporting data" -NoConsole
Invoke-Expression $get
Update-XMLData
Write-Log -Level Info -Message "Importing updated XML data into new policy `"$TargetPolicyName`"" -NoConsole
Invoke-Expression $set

$Successful = "Get-Cs$PolicyType -Identity $TargetPolicyName"
if (Invoke-Expression $Successful -ErrorAction SilentlyContinue){
  Write-Log -Level Info -Message "Target policy `"$TargetPolicyName`" successfully created" -NoConsole
}else{
  Write-Log -Level Error -Message "Target policy `"$TargetPolicyName`" NOT created" -NoConsole
}

Remove-Item "$ExportFolder\$ExportFile" -Force

# SIG # Begin signature block
# MIIcdgYJKoZIhvcNAQcCoIIcZzCCHGMCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUeP2JUzFLAkf8Rem/beu0/63i
# pfCgghelMIIFLjCCBBagAwIBAgIQA6cFj2F2VqaerCviGAu9jTANBgkqhkiG9w0B
# AQsFADByMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFz
# c3VyZWQgSUQgQ29kZSBTaWduaW5nIENBMB4XDTE2MTAxMjAwMDAwMFoXDTE3MTIy
# MTEyMDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBT
# dGVybGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYD
# VQQDExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB
# CgKCAQEAvVNbtzSVPbpRQfYrhU0lPzCnmYKS2DFpgrL8gI8qUsyojdQJRjF8q5YQ
# uIi92Uwl/hxWcvCf0dU/EbzRZudOr8tsbkLA6x1R7O5Fsd6Q2fwMN4YmhFDKUcxf
# Gjy1byOdEhSQZqUc4xp6T+prcIi1j4EDSp5R0TQSRiKUK5HoqanhRyGfqJjkEOep
# KB9soxNVkB+jTXsIjX1AeuUJZS/m+1rzWf75laoid+rna7KQtJO+DkVNSa96tTH2
# 1Fs/C78UzWBLA5I2gQAgEA3alkMzB7rIr34Grt251O+yMSyCcvjb70AVcKohoS7n
# 2Zk522EK3jJw3F8euSWMbXNegJtPjwIDAQABo4IBxTCCAcEwHwYDVR0jBBgwFoAU
# WsS5eyoKo6XqcQPAYPkt9mV1DlgwHQYDVR0OBBYEFE+s6yTjN7ocNskZ2gxbJ3Ul
# xBo6MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzB3BgNVHR8E
# cDBuMDWgM6Axhi9odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vc2hhMi1hc3N1cmVk
# LWNzLWcxLmNybDA1oDOgMYYvaHR0cDovL2NybDQuZGlnaWNlcnQuY29tL3NoYTIt
# YXNzdXJlZC1jcy1nMS5jcmwwTAYDVR0gBEUwQzA3BglghkgBhv1sAwEwKjAoBggr
# BgEFBQcCARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAIBgZngQwBBAEw
# gYQGCCsGAQUFBwEBBHgwdjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNl
# cnQuY29tME4GCCsGAQUFBzAChkJodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRTSEEyQXNzdXJlZElEQ29kZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/
# BAIwADANBgkqhkiG9w0BAQsFAAOCAQEAL3G7W2y46ZvBae5m6LR+rZu1yA1kkmk5
# 55rTY9lA9xCd0jrhY3cPzwKxGFy82surJeWDWbDIao7wqGXIfDrDgFahdM63btdl
# LWJacdfrM7l+82Zx22R30P/K5sVmk1UXr9eyOPzrYMiemUVru3xMNSZ4Md7lHrfb
# CODsEGjs+zWo4eqXWcnnOCxMd1cpr4PvKPonUhMAhCyFDD5TDzZDQIbHBDIvfb8j
# rg/cvZ2PArf5QZpLp5CiSGnGpC+O9rZhYbsOmcJVFMWMM0pMEw46T9HXHxHMfDC1
# oYTp8Xyc/JxiQ+084oNoGng4esFZIb18cJFXDfR0f4ie0pJTMvanqjCCBTAwggQY
# oAMCAQICEAQJGBtf1btmdVNDtW+VUAgwDQYJKoZIhvcNAQELBQAwZTELMAkGA1UE
# BhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2lj
# ZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4X
# DTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEyMDAwMFowcjELMAkGA1UEBhMCVVMxFTAT
# BgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEx
# MC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1cmVkIElEIENvZGUgU2lnbmluZyBD
# QTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAPjTsxx/DhGvZ3cH0wsx
# SRnP0PtFmbE620T1f+Wondsy13Hqdp0FLreP+pJDwKX5idQ3Gde2qvCchqXYJawO
# eSg6funRZ9PG+yknx9N7I5TkkSOWkHeC+aGEI2YSVDNQdLEoJrskacLCUvIUZ4qJ
# RdQtoaPpiCwgla4cSocI3wz14k1gGL6qxLKucDFmM3E+rHCiq85/6XzLkqHlOzEc
# z+ryCuRXu0q16XTmK/5sy350OTYNkO/ktU6kqepqCquE86xnTrXE94zRICUj6whk
# PlKWwfIPEvTFjg/BougsUfdzvL2FsWKDc0GCB+Q4i2pzINAPZHM8np+mM6n9Gd8l
# k9ECAwEAAaOCAc0wggHJMBIGA1UdEwEB/wQIMAYBAf8CAQAwDgYDVR0PAQH/BAQD
# AgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMHkGCCsGAQUFBwEBBG0wazAkBggrBgEF
# BQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEMGCCsGAQUFBzAChjdodHRw
# Oi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0Eu
# Y3J0MIGBBgNVHR8EejB4MDqgOKA2hjRodHRwOi8vY3JsNC5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsMDqgOKA2hjRodHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsME8GA1UdIARI
# MEYwOAYKYIZIAYb9bAACBDAqMCgGCCsGAQUFBwIBFhxodHRwczovL3d3dy5kaWdp
# Y2VydC5jb20vQ1BTMAoGCGCGSAGG/WwDMB0GA1UdDgQWBBRaxLl7KgqjpepxA8Bg
# +S32ZXUOWDAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzANBgkqhkiG
# 9w0BAQsFAAOCAQEAPuwNWiSz8yLRFcgsfCUpdqgdXRwtOhrE7zBh134LYP3DPQ/E
# r4v97yrfIFU3sOH20ZJ1D1G0bqWOWuJeJIFOEKTuP3GOYw4TS63XX0R58zYUBor3
# nEZOXP+QsRsHDpEV+7qvtVHCjSSuJMbHJyqhKSgaOnEoAjwukaPAJRHinBRHoXpo
# aK+bp1wgXNlxsQyPu6j4xRJon89Ay0BEpRPw5mQMJQhCMrI2iiQC/i9yfhzXSUWW
# 6Fkd6fp0ZGuy62ZD2rOwjNXpDd32ASDOmTFjPQgaGLOBm0/GkxAG/AeB+ova+YJJ
# 92JuoVP6EpQYhS6SkepobEQysmah5xikmmRR7zCCBmowggVSoAMCAQICEAMBmgI6
# /1ixa9bV6uYX8GYwDQYJKoZIhvcNAQEFBQAwYjELMAkGA1UEBhMCVVMxFTATBgNV
# BAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8G
# A1UEAxMYRGlnaUNlcnQgQXNzdXJlZCBJRCBDQS0xMB4XDTE0MTAyMjAwMDAwMFoX
# DTI0MTAyMjAwMDAwMFowRzELMAkGA1UEBhMCVVMxETAPBgNVBAoTCERpZ2lDZXJ0
# MSUwIwYDVQQDExxEaWdpQ2VydCBUaW1lc3RhbXAgUmVzcG9uZGVyMIIBIjANBgkq
# hkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAo2Rd/Hyz4II14OD2xirmSXU7zG7gU6mf
# H2RZ5nxrf2uMnVX4kuOe1VpjWwJJUNmDzm9m7t3LhelfpfnUh3SIRDsZyeX1kZ/G
# FDmsJOqoSyyRicxeKPRktlC39RKzc5YKZ6O+YZ+u8/0SeHUOplsU/UUjjoZEVX0Y
# hgWMVYd5SEb3yg6Np95OX+Koti1ZAmGIYXIYaLm4fO7m5zQvMXeBMB+7NgGN7yfj
# 95rwTDFkjePr+hmHqH7P7IwMNlt6wXq4eMfJBi5GEMiN6ARg27xzdPpO2P6qQPGy
# znBGg+naQKFZOtkVCVeZVjCT88lhzNAIzGvsYkKRrALA76TwiRGPdwIDAQABo4ID
# NTCCAzEwDgYDVR0PAQH/BAQDAgeAMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAww
# CgYIKwYBBQUHAwgwggG/BgNVHSAEggG2MIIBsjCCAaEGCWCGSAGG/WwHATCCAZIw
# KAYIKwYBBQUHAgEWHGh0dHBzOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMwggFkBggr
# BgEFBQcCAjCCAVYeggFSAEEAbgB5ACAAdQBzAGUAIABvAGYAIAB0AGgAaQBzACAA
# QwBlAHIAdABpAGYAaQBjAGEAdABlACAAYwBvAG4AcwB0AGkAdAB1AHQAZQBzACAA
# YQBjAGMAZQBwAHQAYQBuAGMAZQAgAG8AZgAgAHQAaABlACAARABpAGcAaQBDAGUA
# cgB0ACAAQwBQAC8AQwBQAFMAIABhAG4AZAAgAHQAaABlACAAUgBlAGwAeQBpAG4A
# ZwAgAFAAYQByAHQAeQAgAEEAZwByAGUAZQBtAGUAbgB0ACAAdwBoAGkAYwBoACAA
# bABpAG0AaQB0ACAAbABpAGEAYgBpAGwAaQB0AHkAIABhAG4AZAAgAGEAcgBlACAA
# aQBuAGMAbwByAHAAbwByAGEAdABlAGQAIABoAGUAcgBlAGkAbgAgAGIAeQAgAHIA
# ZQBmAGUAcgBlAG4AYwBlAC4wCwYJYIZIAYb9bAMVMB8GA1UdIwQYMBaAFBUAEisT
# mLKZB+0e36K+Vw0rZwLNMB0GA1UdDgQWBBRhWk0ktkkynUoqeRqDS/QeicHKfTB9
# BgNVHR8EdjB0MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNl
# cnRBc3N1cmVkSURDQS0xLmNybDA4oDagNIYyaHR0cDovL2NybDQuZGlnaWNlcnQu
# Y29tL0RpZ2lDZXJ0QXNzdXJlZElEQ0EtMS5jcmwwdwYIKwYBBQUHAQEEazBpMCQG
# CCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQQYIKwYBBQUHMAKG
# NWh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRENB
# LTEuY3J0MA0GCSqGSIb3DQEBBQUAA4IBAQCdJX4bM02yJoFcm4bOIyAPgIfliP//
# sdRqLDHtOhcZcRfNqRu8WhY5AJ3jbITkWkD73gYBjDf6m7GdJH7+IKRXrVu3mrBg
# JuppVyFdNC8fcbCDlBkFazWQEKB7l8f2P+fiEUGmvWLZ8Cc9OB0obzpSCfDscGLT
# Ykuw4HOmksDTjjHYL+NtFxMG7uQDthSr849Dp3GdId0UyhVdkkHa+Q+B0Zl0DSbE
# Dn8btfWg8cZ3BigV6diT5VUW8LsKqxzbXEgnZsijiwoc5ZXarsQuWaBh3drzbaJh
# 6YoLbewSGL33VVRAA5Ira8JRwgpIr7DUbuD0FAo6G+OPPcqvao173NhEMIIGzTCC
# BbWgAwIBAgIQBv35A5YDreoACus/J7u6GzANBgkqhkiG9w0BAQUFADBlMQswCQYD
# VQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGln
# aWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJvb3QgQ0Ew
# HhcNMDYxMTEwMDAwMDAwWhcNMjExMTEwMDAwMDAwWjBiMQswCQYDVQQGEwJVUzEV
# MBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29t
# MSEwHwYDVQQDExhEaWdpQ2VydCBBc3N1cmVkIElEIENBLTEwggEiMA0GCSqGSIb3
# DQEBAQUAA4IBDwAwggEKAoIBAQDogi2Z+crCQpWlgHNAcNKeVlRcqcTSQQaPyTP8
# TUWRXIGf7Syc+BZZ3561JBXCmLm0d0ncicQK2q/LXmvtrbBxMevPOkAMRk2T7It6
# NggDqww0/hhJgv7HxzFIgHweog+SDlDJxofrNj/YMMP/pvf7os1vcyP+rFYFkPAy
# IRaJxnCI+QWXfaPHQ90C6Ds97bFBo+0/vtuVSMTuHrPyvAwrmdDGXRJCgeGDboJz
# PyZLFJCuWWYKxI2+0s4Grq2Eb0iEm09AufFM8q+Y+/bOQF1c9qjxL6/siSLyaxhl
# scFzrdfx2M8eCnRcQrhofrfVdwonVnwPYqQ/MhRglf0HBKIJAgMBAAGjggN6MIID
# djAOBgNVHQ8BAf8EBAMCAYYwOwYDVR0lBDQwMgYIKwYBBQUHAwEGCCsGAQUFBwMC
# BggrBgEFBQcDAwYIKwYBBQUHAwQGCCsGAQUFBwMIMIIB0gYDVR0gBIIByTCCAcUw
# ggG0BgpghkgBhv1sAAEEMIIBpDA6BggrBgEFBQcCARYuaHR0cDovL3d3dy5kaWdp
# Y2VydC5jb20vc3NsLWNwcy1yZXBvc2l0b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIB
# Vh6CAVIAQQBuAHkAIAB1AHMAZQAgAG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkA
# ZgBpAGMAYQB0AGUAIABjAG8AbgBzAHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAA
# dABhAG4AYwBlACAAbwBmACAAdABoAGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAA
# LwBDAFAAUwAgAGEAbgBkACAAdABoAGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIA
# dAB5ACAAQQBnAHIAZQBlAG0AZQBuAHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQA
# IABsAGkAYQBiAGkAbABpAHQAeQAgAGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIA
# cABvAHIAYQB0AGUAZAAgAGgAZQByAGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUA
# bgBjAGUALjALBglghkgBhv1sAxUwEgYDVR0TAQH/BAgwBgEB/wIBADB5BggrBgEF
# BQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBD
# BggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0
# QXNzdXJlZElEUm9vdENBLmNydDCBgQYDVR0fBHoweDA6oDigNoY0aHR0cDovL2Ny
# bDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNybDA6oDig
# NoY0aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9v
# dENBLmNybDAdBgNVHQ4EFgQUFQASKxOYspkH7R7for5XDStnAs0wHwYDVR0jBBgw
# FoAUReuir/SSy4IxLVGLp6chnfNtyA8wDQYJKoZIhvcNAQEFBQADggEBAEZQPsm3
# KCSnOB22WymvUs9S6TFHq1Zce9UNC0Gz7+x1H3Q48rJcYaKclcNQ5IK5I9G6OoZy
# rTh4rHVdFxc0ckeFlFbR67s2hHfMJKXzBBlVqefj56tizfuLLZDCwNK1lL1eT7EF
# 0g49GqkUW6aGMWKoqDPkmzmnxPXOHXh2lCVz5Cqrz5x2S+1fwksW5EtwTACJHvzF
# ebxMElf+X+EevAJdqP77BzhPDcZdkbkPZ0XN1oPt55INjbFpjE/7WeAjD9KqrgB8
# 7pxCDs+R1ye3Fu4Pw718CqDuLAhVhSK46xgaTfwqIa1JMYNHlXdx3LEbS0scEJx3
# FMGdTy9alQgpECYxggQ7MIIENwIBATCBhjByMQswCQYDVQQGEwJVUzEVMBMGA1UE
# ChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYD
# VQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgQ29kZSBTaWduaW5nIENBAhAD
# pwWPYXZWpp6sK+IYC72NMAkGBSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKAC
# gAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsx
# DjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBRZjO+YkiigfdntpASUJYZL
# wB03ODANBgkqhkiG9w0BAQEFAASCAQAAJpDd5OWJwnWtAfYBczd2GTd844GqadyN
# tug/2p1H/pgr99if0oIOjnxic0drOJzoAINATcF1vSYXZTDsBpD9st+S/NNGXQMu
# fsuMQkSdJjaAu6gv45eTooOw1hEIQofr2WY6OplEcUye9W5VmWbc3PFrkTItxdf6
# 458045XIwfXUDiTcsbZl9iNxEwmWwVjuGkcmzc7nTd5aO/p6UOLq4lpCdZKbX9EF
# PF83RolKeUwaRgO8LCH9skPDAfd7PJnHNhs7dC0UibgJzDRMacP0C/IPH/64hngb
# CZWSv5bPIZbnwg+wIjroHFoGQgV/iG7VaH6eszyWqb3rOAazMqkJoYICDzCCAgsG
# CSqGSIb3DQEJBjGCAfwwggH4AgEBMHYwYjELMAkGA1UEBhMCVVMxFTATBgNVBAoT
# DERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8GA1UE
# AxMYRGlnaUNlcnQgQXNzdXJlZCBJRCBDQS0xAhADAZoCOv9YsWvW1ermF/BmMAkG
# BSsOAwIaBQCgXTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJ
# BTEPFw0xNzAyMDMwOTEyNDFaMCMGCSqGSIb3DQEJBDEWBBSL78OBpu6kTD2qe/ct
# tPVVTfBPsDANBgkqhkiG9w0BAQEFAASCAQAOzO98BCnCCZwm3O9XNHr5worp8wQR
# 8T2EU4reFruYadZrGsQWmRsdYEqKbsRm5Q6ubnzu9EnkjBD16nwWHUUmbEhLXj4J
# Ic/Ud9lwLC5kDBGGEB2+VLipj7X4JwNGQ04SV4XOYKltOdSSet0tZ5l78qLp5qRV
# j/lDPlBHhKOPvvW7qRKX8tyLyJPiMPm1zGIl17QfPN6VO6wkTvVV1xKf5y1JiCeK
# cXUCi//XKCb45kYchMFmwidm9U+jrVd1psQadnGbhWRcWNFE8ZuTarYistFzPG/n
# WMHCkWOPDe7or595W42bKSUDp9RcnFpAC4CQTVka8M6f9wwcbBvcpng0
# SIG # End signature block
